@@ -23,6 +23,7 @@ enum PipewireMessage {
     NodeAdded {
         id: u32,
         name: String,
+        node_type: Option<NodeType>,
     },
     PortAdded {
         id: u32,
@@ -55,6 +56,12 @@ enum PipewireMessage {
     },
 }
 
+#[derive(Debug, Clone)]
+pub enum NodeType {
+    Input,
+    Output,
+}
+
 #[derive(Debug, Copy, Clone)]
 pub enum MediaType {
     Audio,
